"""
Udiskie unmount utilities.
"""
__all__ = [
    'unmount_device', 'lock_device', 'remove_device', 'lock_slave',
    'unmount', 'unmount_all']

from udiskie.mount import unmount_device, lock_device, remove_device, lock_slave, unmount, unmount_all

